# IBIS.pm
#
# Eugene Eric Kim <eekim@eekim.com>
# http://www.eekim.com/software/perlIBIS/
#
# $Id: IBIS.pm,v 1.6 2002/02/26 09:24:55 eekim Exp $
#
# Copyright (c) Eugene Eric Kim 2001-2002.  All rights reserved.
# See COPYING for licensing terms.

package IBIS;

use 5.6.1;  # almost certainly works w/ earlier versions; just not tested
$VERSION = '0.2';

=head1 NAME

IBIS - Interface for Issue Based Information Systems dialog maps.

=head1 SYNOPSIS

=head2 Example 1

 use IBIS;

 $dialogMap = new IBIS;

 # create nodes and links
 $node1Id = $dialogMap->createNode(
     type=>'Issue',
     label=>'What is the meaning of life?');
 $node2Id = $dialogMap->createNode(
     type=>'Argument',
     label=>'43');
 $node3Id = $dialogMap->createNode(
     type=>'Position',
     label=>'Douglas Adams is always right');
 $link1Id = $dialogMap->createLink('Response', $node2Id, $node1Id);
 $link2Id = $dialogMap->createLink('Oppose', $node3Id, $node2Id);

 # modify node and link info
 $dialogMap->getNode($node2Id)->label('42');
 $dialogMap->getLink($link2Id)->type('Support');

=head2 Example 2

 use IBIS;

 $dialogMap = new IBIS;

 # read QuestMap file file.txt
 $dialogMap->readFile('QuestMap', 'file.qm');

 # iterate through nodes
 $node = $dialogMap->firstNode;
 while ($node) {
   print $node->label . "\n";
   $node = $dialogMap->nextNode;
 }

 # write XML file
 $dialogMap->writeFile('XML', 'file.xml');

 # write PNG file
 $dialogMap->writeFile('PNG','file.png');

=head2 Example 3

 use IBIS;

 $dialogMap = new IBIS;

 # read QuestMap file file.txt
 $dialogMap->readFile('QuestMap', 'file.qm');

 # get independent nodes
 my @nodeIds = $dialogMap->getIndependentNodes;
 # print these nodes
 foreach my $nodeId (@nodeIds) {
   my $node = $dialogMap->getNode($nodeId);
   print $node->type . ': ' . $node->label . "\n";
   # print ideas responding to these issues
   my @childIds = $dialogMap->getChildren($nodeId);
   foreach my $childId (@childIds) {
     my $node = $dialogMap->getNode($childId);
     print '  ' . $node->type . ': ' . $node->label . "\n";
   }
 }

=head1 DESCRIPTION

Issue-Based Information System (IBIS) is a methodology for discussing
and exploring "wicked problems," problems that are not well understood
and that have no straightforward solution.  Invented by Horst Rittel
and colleagues in the 1970s, IBIS provides a simple grammar for
mapping conversations.  The grammar consists of questions (issues),
possible solutions (ideas), and arguments (pros and cons for or
against an idea).

In the 1980s, Jeff Conklin and others developed software for
graphically mapping and representing IBIS conversations.  This
software eventually evolved into a Windows program called QuestMap.

This module provides a graph-like data structure for creating and
manipulating IBIS conversations.  It was designed primarily to convert
QuestMap files into other file formats, but it could conceivably be
used as the core data structure for a full-fledged IBIS application.

This module includes a simple driver architecture to support new file
formats.  Included in this distribution are modules for reading and/or
writing to QuestMap, XML, and PNG.

For more information on the IBIS methodology and QuestMap, see:

    http://www.gdss.com/wp/IBIS.htm
    http://www.softbicycle.com/questmp.html

=cut

use strict;
use vars qw($VERSION);
use IBIS::Node;
use IBIS::Link;

### public methods

=head1 METHODS

=head2 new

Constructor.

 pre:
   None.
 post:
   Returns an empty, "blessed" object.

=cut

sub new {
  my $this = shift;
  my $self = {};

  $self->{'nodes'} = ();
  $self->{'links'} = ();

  # indices
  $self->{'index_children'} = ();
  $self->{'index_parents'} = ();

  bless($self, $this);
  return $self;
}

=head2 createNode(%nodeHash)

Creates a new node, and adds it to the map.  All of the parameters are
optional.  If a nonunique 'id' is passed, then uses a different,
unique ID.  If 'id' is not passed at all, then creates and uses a
unique ID.  Calling this function also resets the map's node iterator.

 pre:
   Takes a hash as a parameter:
     { id => int
       type => Issue|Response|Argument|Pro|Con|Position|
               Note|Decision|Reference|MapView|ListView
       label => string
       detail => string
       author => string
       url => string
       creationDate => date
       modificationDate => date
       xPos => int   # for map views
       yPos => int
       pos => int    # for list views
     }
 post:
   Adds node to the map, returns its ID, and resets node iterator.

=cut

sub createNode {
  my $this = shift;
  my %nodeHash = @_;
  my $node = new IBIS::Node;
  my ($id);

  # choose node ID
  if ($nodeHash{'id'}) {
    # check if node ID is already in use
    if ($this->{'nodes'}->{$nodeHash{'id'}}) {
      $id = $this->getUniqueNodeId;
    }
    else {
      $id = $nodeHash{'id'};
    }
  }
  else {
    $id = &_getUniqueNodeId;
  }
  # populate node fields
  $node->id($id);
  $node->type($nodeHash{'type'}) if $nodeHash{'type'};
  $node->label($nodeHash{'label'}) if $nodeHash{'label'};
  $node->detail($nodeHash{'detail'}) if $nodeHash{'detail'};
  $node->author($nodeHash{'author'}) if $nodeHash{'author'};
  $node->url($nodeHash{'url'}) if $nodeHash{'url'};
  $node->creationDate($nodeHash{'creationDate'} ? $nodeHash{'creationDate'} :
    time);
  $node->modificationDate($nodeHash{'modificationDate'} ?
    $nodeHash{'modificationDate'} : time);
  $node->xPos($nodeHash{'xPos'}) if defined $nodeHash{'xPos'};
  $node->yPos($nodeHash{'yPos'}) if defined $nodeHash{'yPos'};
  $node->pos($nodeHash{'pos'}) if defined $nodeHash{'pos'};
  # add node to object
  $this->{'nodes'}->{$id} = $node;
  # reset iterator
  keys %{$this->{'nodes'}};

  return $id;
}

=head2 getNode($id)

Returns node with ID id.

 pre:
   Node ID.
 post:
   Returns an IBIS::Node object.

=cut

sub getNode {
  my $this = shift;
  my $id = shift;

  return $this->{'nodes'}->{$id};
}

=head2 firstNode

Returns "first" node.  Can be used in conjunction with nextNode() to
iterate through all of the nodes.  Works by resetting the node
iterator, and returning the first node.

 pre:
   None.
 post:
   Resets the node iterator, and returns the first IBIS::Node object.

=cut

sub firstNode {
  my $this = shift;

  # reset iterator
  keys %{$this->{'nodes'}};
  # get first node
  my $i = each %{$this->{'nodes'}};
  return $this->{'nodes'}->{$i};
}

=head2 nextNode

Returns the next node.  Can be used in conjunction with firstNode to
iterate through all the nodes:

 $dmap = new IBIS;
 $node = $dmap->firstNode;
 while ($node) {
   # do something here
   $node = $node->nextNode;
 }

 pre:
   None.
 post:
   Increments the node iterator, and returns the IBIS::Node object.

=cut

sub nextNode {
  my $this = shift;

  my $i = each %{$this->{'nodes'}};
  return $this->{'nodes'}->{$i};
}

=head2 getUniqueNodeId

Returns a unique node ID.

 pre:
   None.
 post:
   Returns a unique node ID.

=cut

sub getUniqueNodeId {
  my $this = shift;
  my $id = 1;

  my %ids = map { $_ => 1 } keys %{$this->{'nodes'}};
  while ($ids{$id}) {
    $id++;
  }
  return $id;
}

=head2 createLink($type, $fromNodeId, $toNodeId, $id)

Creates a new link, and adds it to the map.  To and from nodes must
exist in the map.  Also resets the link iterator.

 pre:
   $type -- RespondsTo|ObjectsTo|Supports|ExpandsOn|Decides|Specializes|About
   Nodes with IDs $fromNodeId and $toNodeId must exist in the map.
   $id (the link's ID) is optional.
 post:
   Returns link ID, and resets link iterator.

=cut

sub createLink {
  my $this = shift;
  my ($type, $fromNodeId, $toNodeId, $id) = @_;
  my $link = new IBIS::Link;

  # check for valid node IDs
  if (!($this->{'nodes'}->{$fromNodeId} && $this->{'nodes'}->{$toNodeId})) {
    return undef;
  }
  # choose link ID
  if (!$id || ($id && $this->{'links'}->{$id})) {
      $id = $this->getUniqueLinkId;
  }
  # populate link fields
  $link->id($id);
  $link->type($type);
  $link->fromNodeId($fromNodeId);
  $link->toNodeId($toNodeId);

  # add link to object
  $this->{'links'}->{$id} = $link;

  # reset iterator
  keys %{$this->{'links'}};

  # update indices
  # Push the link rather than the appropriate node ID, because you
  # want to have the link->type handy.
  push @{$this->{'index_children'}->{$toNodeId}}, $link;
  push @{$this->{'index_parents'}->{$fromNodeId}}, $link;

  return $id;
}

=head2 getLink($id)

Returns link with ID id.

 pre:
   Link ID.
 post:
   Returns IBIS::Link object.

=cut

sub getLink {
  my $this = shift;
  my $id = shift;

  return $this->{'links'}->{$id};
}

=head2 firstLink

Returns "first" link.  Can be used in conjunction with nextLink() to
iterate through all the links.  (See firstNode() and nextNode() for an
example.)

 pre:
   None.
 post:
   Returns the "first" IBIS::Link object.

=cut

sub firstLink {
  my $this = shift;

  # reset iterator
  keys %{$this->{'links'}};
  # get first node
  my $i = each %{$this->{'links'}};
  return $this->{'links'}->{$i};
}

=head2 nextLink

Returns the next link.

 pre:
   None.
 post:
   Returns the next IBIS::Link object.

=cut

sub nextLink {
  my $this = shift;

  my $i = each %{$this->{'links'}};
  return $this->{'links'}->{$i};
}

=head2 getLinksToParents($childNodeId)

Given a node ID, returns all links with that node as a child.

 pre:
   Child's node ID.
 post:
   Returns a list of links.

=cut

sub getLinksToParents {
  my $this = shift;
  my $childNodeId = shift;

  if ($this->{'index_parents'}->{$childNodeId}) {
    return @{$this->{'index_parents'}->{$childNodeId}};
  }
  else {
    return ();
  }
}

=head2 getUniqueLinkId

Returns a unique link ID.

 pre:
   None.
 post:
   Returns a unique link ID.

=cut

sub getUniqueLinkId {
  my $this = shift;
  my $id = -1;

  my %ids = map { $_ => 1 } keys %{$this->{'links'}};
  while ($ids{$id}) {
    $id--;
  }
  return $id;
}

=head2 getIndependentNodes()

Returns all nodes that are not pointing to anything else.

 pre:
   None.
 post:
   Returns a list of node IDs.

=cut

sub getIndependentNodes {
  my $this = shift;
  my @independentNodes;

  my @allNodeIds = keys %{$this->{'nodes'}};
  foreach my $nodeId (@allNodeIds) {
    if (!$this->{'index_parents'}->{$nodeId}) { # node has no parent
      push @independentNodes, $nodeId;
    }
  }
  if (@independentNodes) { # sort based on pos or yPos
    if (defined $this->getNode($independentNodes[0])->pos) {
      return sort { $this->getNode($a)->pos <=> $this->getNode($b)->pos }
        @independentNodes;
    }
    else {
      return sort { $this->getNode($a)->yPos <=> $this->getNode($b)->yPos }
        @independentNodes;
    }
  }
  return @independentNodes;
}

=head2 getChildren($parentNodeId)

Given a node ID, returns all IDs of that node's children.

 pre:
   Parent's node ID.
 post:
   Returns a list of node IDs.

=cut

sub getChildren {
  my $this = shift;
  my $parentNodeId = shift;
  my (@links, @nodeIds);

  if ($this->{'index_children'}->{$parentNodeId}) {
    @links = @{$this->{'index_children'}->{$parentNodeId}};
    foreach my $link (@links) {
      my $node = $this->getNode($link->fromNodeId);
      if ( (($node->type eq 'Argument') || ($node->type eq 'Pro') ||
           ($node->type eq 'Con')) && ($link->type eq 'Supports') ) {
        $node->type('Pro');
      }
      elsif ( (($node->type eq 'Argument') || ($node->type eq 'Pro') ||
           ($node->type eq 'Con')) && ($link->type eq 'ObjectsTo') ) {
        $node->type('Con');
      }
      push @nodeIds, $node->id;
    }
    if (defined $this->getNode($nodeIds[0])->pos) {
      return sort { $this->getNode($a)->pos <=> $this->getNode($b)->pos }
        @nodeIds;
    }
    else {
      return sort { $this->getNode($a)->yPos <=> $this->getNode($b)->yPos }
        @nodeIds;
    }
  }
  else {
    return ();
  }
}

=head2 readFile($fileFormat, $fileName)

Reads the file fileName of type fileFormat, and creates a
corresponding IBIS map.  There must be a driver of type
IBIS::Driver::$fileFormat in order to read the file.  See the
IBIS::Driver POD for more information.

Included in this distribution are:

    IBIS::Driver::PNG
    IBIS::Driver::QuestMap
    IBIS::Driver::XML

 pre:
   $fileFormat must correspond to one of the installed drivers.
 post:
   The IBIS object will be populated with the contents of $fileName.

=cut

sub readFile {
  my $this = shift;
  my ($fileFormat, $fileName) = @_;

  # load appropriate driver
  eval "use IBIS::Driver::$fileFormat;";
  if ($@) {
    print &_driverError($@);
    return;
  }
  # does file exist?
  if (!-e $fileName) {
    print "File $fileName does not exist!\n";
    return;
  }
  # parse file
  my $evalString = '&IBIS::Driver::' . $fileFormat .
    "::readFile(\$this, '$fileName');";
  eval $evalString;
  print &_driverError($@) if ($@);
}

=head2 writeFile($fileFormat, $fileName)

Writes the contents of the object to $fileName of type $fileFormat,
where $fileFormat corresponds to one of the installed drivers.  See
readFile() for more information.

 pre:
   $fileFormat must correspond to one of the installed drivers.
 post:
   Writes object's content to $fileName of type $fileFormat.

=cut

sub writeFile {
  my $this = shift;
  my ($fileFormat, $fileName) = @_;

  # load appropriate driver
  eval "use IBIS::Driver::$fileFormat;";
  if ($@) {
    print &_driverError($@);
    return;
  }
  # does file exist?
  if (-e $fileName) {
    print "File $fileName already exists.  Overwrite? ([y]/n) ";
    my $response = lc(<STDIN>);
    chomp $response;
    $response = lc($response);
    if ( ($response ne '') && !($response =~ /^y/) ) {
      return;
    }
  }
  # serialize and write file
  my $evalString = '&IBIS::Driver::' . $fileFormat .
    "::writeFile(\$this, '$fileName');";
  eval $evalString;
  print &_driverError($@);
}

### private methods

sub _driverError {
  my $errorMsg = shift;

  if ($errorMsg =~ /^\s*$/) {
    return;
  }
  elsif ($errorMsg =~ /^Can\'t locate IBIS\/Driver\/(\w+\.pm)/) {
    my $fileFormat = $1;
    $fileFormat =~ s/\.pm$//;
    return "Error: IBIS::Driver::$fileFormat driver not installed.\n";
  }
  elsif ($errorMsg =~ /^Undefined subroutine \&IBIS::Driver::(\w+)::(\w+)/) {
    if ($2 eq 'writeFile') {
      return "Error: IBIS::Driver::$1 does not support writing files.\n";
    }
    else {
      return "Error: IBIS::Driver::$1 does not support reading files.\n";
    }
  }
  else {
    return "Error: $@\n";
  }
}

1;

=head1 AUTHOR

Eugene Eric Kim <eekim@eekim.com>

=cut
