# IBIS::Driver::HTMLTagged.pm
#
# Eugene Eric Kim <eekim@eekim.com>
# http://www.eekim.com/software/perlIBIS/
#
# $Id: HTMLTagged.pm,v 1.1 2002/02/26 09:21:22 eekim Exp $
#
# Copyright (c) Eugene Eric Kim 2001-2002.  All rights reserved.
# See COPYING for licensing terms.

package IBIS::Driver::HTMLTagged;

use 5.6.1;  # almost certainly works w/ earlier versions; just not tested
$VERSION = '0.2';

=head1 NAME

IBIS::Driver::HTMLTagged - HTML driver for IBIS.pm.

=head1 DESCRIPTION

Called by IBIS.pm.

Essentially the same as HTML.pm, except &_parseDetail has been
modified to account for special tags used in QuestMap.

Should be rewritten to inherit from HTML.pm, overwriting a public
method version of &_parseDetail.

=cut

use strict;
use vars qw($VERSION);
use IO::File;

=head1 METHODS

=head2 writeFile()

Outputs map in ASCII text.

 pre:
   $ibisObject must be a valid IBIS object.
   $fileName
 post:
   None.

=cut

sub writeFile {
  my ($ibisObject, $fileName) = @_;

  my $fh = new IO::File ">$fileName";
  &_htmlStart($fh, $fileName);
  # traverse map, starting w/ independent nodes
  my @nodeIds = $ibisObject->getIndependentNodes;
  foreach my $nodeId (@nodeIds) {
    &_htmlNode($fh, $ibisObject->getNode($nodeId));
    &_traverseMap($nodeId, 1, $ibisObject, $fh);
  }
  &_htmlEnd($fh);
  $fh->close;
}

# private methods

sub _htmlNode {
  my $fh = shift;
  my $node = shift;

  print $fh '<p><a class="nid" href="#nid' . $node->id . '"></a>';
  if ($node->type eq 'Issue') {
    print $fh '<span class="issue">';
  }
  elsif ($node->type eq 'Pro') {
    print $fh '<span class="pro">';
  }
  elsif ($node->type eq 'Con') {
    print $fh '<span class="con">';
  }
  elsif ( ($node->type eq 'MapView') || ($node->type eq 'ListView') ) {
    $node->url(&_readInput('URL for node "' . $node->label . '"?'))
      unless defined $node->url;
  }
  if ($node->url) {
    print $fh '<a href="' . $node->url . '">';
  }
  print $fh &_escapeString($node->label);
  if ($node->url) {
    print $fh '</a>';
  }
  if ( ($node->type eq 'Issue') || ($node->type eq 'Pro') ||
       ($node->type eq 'Con') ) {
    print $fh '</span>';
  }
  print $fh ' &nbsp;&nbsp; <a class="nid" href="#nid' . $node->id . '">(' .
    $node->id . ')</a>';
  if ($node->detail) {
    print $fh "<br>\n<span class=\"detail\">" . &_parseDetail($node->detail)
      . "</span>";
  }
  print $fh "</p>\n";
}

sub _htmlStart {
  my ($fh, $fileName) = @_;

  my $title = &_escapeString(&_readInput('Title?') || $fileName);
  my $ownerName = &_escapeString(&_readInput('Owner name?'));
  my $ownerEmail = &_escapeString(&_readInput('Owner email?'));
  print $fh <<EOM;
<html>
  <head>
    <title>$title</title>
    <style type="text/css">
      body {
        background-color:#FFFFFF;
        padding-bottom: 50em;}
      h1 {text-align: center;}
      p.docinfo {text-align: center;}
      p.footer {text-align: right;}
      div.indented {margin-left: 3em;}
      p {margin-top: 4pt; margin-bottom: 4pt;}
      span.issue { font-weight: bold; }
      span.pro { color: green; }
      span.con { color: red; }
      span.detail {
        color: gray;
        font-family: "Helvetica", "Arial", sans-serif;
        font-style: normal;
        font-size: smaller;}
      a.nid {
        font-family: "Helvetica", "Arial", sans-serif;
        font-style: normal;
        font-weight: bold;
        font-size: x-small;
        text-decoration: none;
        color: #C8A8FF;}
      a.ref { color: gray;}
    </style>
  </head>
  <body>
    <h1>$title</h1>

    <p class="docinfo">$ownerName
    <a href="mailto:$ownerEmail">&lt;$ownerEmail&gt;</a></p>

    <hr>

EOM
}

sub _htmlEnd {
  my $fh = shift;

  print $fh <<EOM;
  <hr>
  <p class="footer"><i>Document created using
  <a href="http://www.eekim.com/software/perlIBIS/">perlIBIS</a>.</i></p>
  </body>
</html>
EOM
}

sub _printSpaces {
  my ($indentLevel, $fh) = @_;

  $indentLevel = $indentLevel * 4;
  while ($indentLevel > 0) {
    print $fh " ";
    $indentLevel--;
  }
}

sub _escapeString {
  my $string = shift;

  $string =~ s/\&/\&amp\;/g;
  $string =~ s/\"/\&quot\;/g;
  $string =~ s/</\&lt\;/g;
  return $string;
}

sub _parseDetail {
  my $string = shift;
  my ($newstring, $refstring);

  chomp $string;
  my @lines = split(/\n/, $string);
  foreach my $line (@lines) {
    if ($line =~ /^\$ref\s*=\s*\(\s*(\d\d\d\d\d\d\d\d)\s*,\s*"([^"]+)"\s*,\s*"([^"]+)"\s*\)\s*$/) {
      my $date = &_convertDate($1);
      my $email = $2;
      my $url = $3;
      $refstring .= '<a class="ref" href="' . $url . '">' .
        "<i>$email</i> ($date)" . "</a>,\n";
    }
    else {
      $newstring .= "$line<br>\n";
    }
  }
  if ($refstring) {
    $refstring =~ s/,\n$//s;
    $newstring .= "<b>References:</b> \n$refstring";
  }
  $newstring =~ s/<br>\n$//s;
  return $newstring;
}

sub _convertDate {
  my $dateNum = shift;
  my %monthName = ('01' => 'Jan', '02' => 'Feb', '03' => 'Mar',
                   '04' => 'Apr', '05' => 'May', '06' => 'Jun',
                   '07' => 'Jul', '08' => 'Aug', '09' => 'Sep',
                   '10' => 'Oct', '11' => 'Nov', '12' => 'Dec');
  my ($monthNum, $day, $year);

  $dateNum =~ /(\d\d\d\d)(\d\d)(\d\d)/;
  $year = $1;
  $monthNum = $2;
  $day = $3;
  $day =~ s/^0//;
  return $monthName{$monthNum} . " $day, $year";
}

sub _readInput {
  my $msg = shift;

  print $msg . ' ';
  my $input = <STDIN>;
  chomp $input;
  if ($input eq '') {
    return undef;
  }
  else {
    return $input;
  }
}

sub _traverseMap {
  my ($nodeId, $indentLevel, $ibisObject, $fh) = @_;

  my @children = $ibisObject->getChildren($nodeId);
  if (@children) {
    print $fh '<div class="indented">' . "\n";
  }
  foreach my $childId (@children) {
    &_htmlNode($fh, $ibisObject->getNode($childId));
    &_traverseMap($childId, $indentLevel + 1, $ibisObject, $fh);
  }
  if (@children) {
    print $fh "</div>\n";
  }
}

1;

=head1 AUTHOR

Eugene Eric Kim <eekim@eekim.com>

=cut
