# IBIS::Driver::OPML.pm
#
# Eugene Eric Kim <eekim@eekim.com>
# http://www.eekim.com/software/perlIBIS/
#
# $Id: OPML.pm,v 1.4 2002/02/26 09:22:13 eekim Exp $
#
# Copyright (c) Eugene Eric Kim 2001-2002.  All rights reserved.
# See COPYING for licensing terms.

package IBIS::Driver::OPML;

use 5.6.1;  # almost certainly works w/ earlier versions; just not tested
$VERSION = '0.2';

=head1 NAME

IBIS::Driver::OPML - OPML driver for IBIS.pm.

=head1 DESCRIPTION

Called by IBIS.pm.

Generates OPML (http://www.opml.org/) files.  I added three attributes
to the outline element -- nid, detail, and url -- corresponding to the
equivalent IBIS node attributes.

=cut

use strict;
use vars qw($VERSION);
use IO::File;

=head1 METHODS

=head2 writeFile()

Outputs map in OPML.

 pre:
   $ibisObject must be a valid IBIS object.
   $fileName
 post:
   None.

=cut

sub writeFile {
  my ($ibisObject, $fileName) = @_;

  my $fh = new IO::File ">$fileName";
  &_opmlStart($fh, $fileName);
  # traverse map, starting w/ independent nodes
  my @nodeIds = $ibisObject->getIndependentNodes;
  foreach my $nodeId (@nodeIds) {
    my $node = $ibisObject->getNode($nodeId);
    &_printSpaces(0, $fh);
    print $fh '<outline type="' . &_escapeString($node->type) . '" ';
    print $fh 'nid="' . &_escapeString($node->id) . '" ' if ($node->id);
    if ( ($node->type eq 'MapView') || ($node->type eq 'ListView') ) {
      my $url = &_readInput('URL for node "' . $node->label . '"?');
      print $fh 'url="' . $url . '" ' if ($url);
    } elsif ($node->url) {
      print $fh 'url="' . &_escapeString($node->url) . '" ';
    }
    print $fh 'detail="' . &_escapeString($node->detail) . '" '
      if ($node->detail);
    print $fh 'text="' . &_escapeString($node->label) . '">' . "\n";
    &_traverseMap($nodeId, 1, $ibisObject, $fh);
    &_printSpaces(0, $fh);
    print $fh "</outline>\n";
  }
  &_opmlEnd($fh);
  $fh->close;
}

# private methods

sub _opmlStart {
  my ($fh, $fileName) = @_;
  my $dateCreated = gmtime;

  my $title = &_readInput('Title?') || $fileName;
  my $ownerName = &_readInput('Owner name?');
  my $ownerEmail = &_readInput('Owner email?');
  print $fh <<EOM;
<?xml version="1.0"?>
<opml version="1.0">
  <head>
    <title>$title</title>
    <dateCreated>$dateCreated GMT</dateCreated>
    <ownerName>$ownerName</ownerName>
    <ownerEmail>$ownerEmail</ownerEmail>
  </head>
  <body>
EOM
}

sub _opmlEnd {
  my $fh = shift;

  print $fh <<EOM;
  </body>
</opml>
EOM
}

sub _escapeString {
  my $string = shift;

  $string =~ s/\&/\&amp\;/g;
  $string =~ s/\"/\&quot\;/g;
  $string =~ s/</\&lt\;/g;
  return $string;
}

sub _readInput {
  my $msg = shift;

  print $msg . ' ';
  my $input = <STDIN>;
  chomp $input;
  if ($input eq '') {
    return undef;
  }
  else {
    return $input;
  }
}

sub _printSpaces {
  my ($indentLevel, $fh) = @_;

  $indentLevel = $indentLevel * 4 + 4;
  while ($indentLevel > 0) {
    print $fh " ";
    $indentLevel--;
  }
}

sub _traverseMap {
  my ($nodeId, $indentLevel, $ibisObject, $fh) = @_;

  my @children = $ibisObject->getChildren($nodeId);
  foreach my $childId (@children) {
    my $node = $ibisObject->getNode($childId);
    &_printSpaces($indentLevel, $fh);
    print $fh '<outline type="' . &_escapeString($node->type) . '" ';
    print $fh 'nid="' . &_escapeString($node->id) . '" ' if ($node->id);
    if ( ($node->type eq 'MapView') || ($node->type eq 'ListView') ) {
      my $url = &_readInput('URL for node "' . $node->label . '"?');
      print $fh 'url="' . $url . '" ' if ($url);
    } elsif ($node->url) {
      print $fh 'url="' . &_escapeString($node->url) . '" ';
    }
    print $fh 'detail="' . &_escapeString($node->detail) . '" '
      if ($node->detail);
    print $fh 'text="' . &_escapeString($node->label) . '">' . "\n";
    &_traverseMap($childId, $indentLevel + 1, $ibisObject, $fh);
    &_printSpaces($indentLevel, $fh);
    print $fh "</outline>\n";
  }
}

1;

=head1 AUTHOR

Eugene Eric Kim <eekim@eekim.com>

=cut
